using System;
using System.Collections;
using System.IO;

namespace DirClean
{
    /// <summary>
    /// Controls the execution of application.
    /// </summary>
    public class Controller
    {
        #region PRIVATE MEMBERS

        private static readonly ArrayList _empties = new ArrayList();
        private static readonly ArrayList _parents = new ArrayList();
        private static bool _confirmEachDelete = Constants.DEFAULT_CONFIRM_EACH_DELETE;
        private static bool _delete = Constants.DEFAULT_DELETE;
        private static bool _hidden = Constants.DEFAULT_HIDDEN;
        private static string _initialDir = string.Empty;
        private static bool _logToConsole = Constants.DEFAULT_CONSOLE;
        private static bool _recursive = Constants.DEFAULT_RECURSE;
        private static bool _silent = Constants.DEFAULT_SILENT;

        #endregion

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        public static void Main(string[] args)
        {
            // parse the options that were passed
            ScanOptions(args);

            if (!Directory.Exists(_initialDir))
            {
                // INITIAL DIR DOESNT EXIST
                if ((_initialDir != null) && (_initialDir.Length > 0))
                {
                    WriteConsoleMessage(_initialDir + " is not a valid path!");
                }
            }
            else
            {
                // INITIAL DIR EXISTS
                CheckFolders(_initialDir, true);

                if (_logToConsole || _confirmEachDelete)
                {
                    WriteConsoleMessage("\nThe following directories are empty");

                    if (_delete)
                    {
                        WriteConsoleMessage(" and will be deleted...\n\n");
                    }
                    else
                    {
                        WriteConsoleMessage("...\n\n");
                    }

                    if (_empties.Count > 0)
                    {
                        foreach (string dir in _empties)
                        {
                            if (_delete)
                            {
                                if (_confirmEachDelete && (_silent == false))
                                {
                                    ConsoleInputHandler.EnableCharMode();

                                    var prompt = new char[1];
                                    WriteConsoleMessage("\t" + dir + " Sure (y/n)? ");
                                    Console.In.Read(prompt, 0, 1);

                                    ConsoleInputHandler.EnableLineMode();

                                    if ((prompt[0] == 'Y') || (prompt[0] == 'y'))
                                    {
                                        try
                                        {
                                            Directory.Delete(dir, true);
                                            WriteConsoleMessage(" DELETED");
                                        }
                                        catch
                                        {
                                        }
                                    }
                                    WriteConsoleMessage("\n");
                                }
                                else
                                {
                                    try
                                    {
                                        Directory.Delete(dir, true);
                                    }
                                    catch
                                    {
                                    }
                                    finally
                                    {
                                        WriteConsoleMessage("\t" + dir + " DELETED");
                                    }
                                }
                            }
                            else
                            {
                                WriteConsoleMessage("\t" + dir + "\n");
                            }
                        }
                    }
                    else
                    {
                        WriteConsoleMessage("\tNone found!");
                    }

                    WriteConsoleMessage("\n\nPress enter to quit...");
                    if (_silent == false)
                    {
                        Console.In.ReadLine();
                    }
                }
                else
                {
                    if (_empties.Count > 0)
                    {
                        foreach (string dir in _empties)
                        {
                            if (_delete)
                            {
                                try
                                {
                                    Directory.Delete(dir, true);
                                }
                                catch
                                {
                                }
                            }
                        }
                    }
                    else
                    {
                        WriteConsoleMessage("\tNone found!");
                    }
                }
            }
        }


        /// <summary>
        /// Checks given folders structure for empty folders.
        /// </summary>
        /// <param name="path">Path of empty folder.</param>
        private static void CheckFolders(string path, bool start)
        {
            string[] dirz;
            string[] filez;
            ArrayList fileList;
            ArrayList dirList;
            ArrayList hiddenList;

            try
            {
                dirz = Directory.GetDirectories(path);
                filez = Directory.GetFiles(path);
                fileList = new ArrayList();
                dirList = new ArrayList();
                hiddenList = new ArrayList();

                for (int x = 0; x < filez.Length; x++)
                {
                    fileList.Add(filez[x]);
                }

                for (int x = 0; x < dirz.Length; x++)
                {
                    dirList.Add(dirz[x]);
                }

                if (_hidden)
                {
                    foreach (string file in fileList)
                    {
                        FileAttributes attrs = File.GetAttributes(file);

                        if (attrs.ToString().IndexOf("Hidden") >= 0)
                        {
                            hiddenList.Add(file);
                        }
                    }

                    foreach (string file in hiddenList)
                    {
                        fileList.Remove(file);
                    }

                    fileList.TrimToSize();
                }

                if ((dirList.Count <= 0) && (fileList.Count <= 0))
                {
                    _empties.Add(path);
                    WriteConsoleMessage(".");
                }
                else
                {
                    if (_recursive)
                    {
                        if (fileList.Count <= 0)
                        {
                            _empties.Add(path);
                            Console.Out.Write(".");

                            foreach (string dir in dirList)
                            {
                                CheckFolders(dir, false);
                            }
                        }
                        else
                        {
                            foreach (string dir in dirList)
                            {
                                CheckFolders(dir, false);
                            }

                            if (start == false)
                            {
                                GetParents(path);
                            }

                            foreach (string parent in _parents)
                            {
                                _empties.Remove(parent);
                            }

                            _empties.TrimToSize();
                            _empties.Sort();
                        }
                    }
                }
            }
            catch
            {
            }
        }


        /// <summary>
        /// Gets the parent path of the given folder.
        /// </summary>
        /// <param name="path">Parent path.</param>
        private static void GetParents(string path)
        {
            DirectoryInfo parentInfo = Directory.GetParent(path);

            string parent = parentInfo.ToString();

            if (!_parents.Contains(parent))
            {
                _parents.Add(parent);
            }

            try
            {
                GetParents(parent);
            }
            catch
            {
            }
        }


        /// <summary>
        /// Parses the command line params.
        /// </summary>
        /// <param name="args">string array of args</param>
        private static void ScanOptions(string[] args)
        {
            int argIndex = 0;
            int argCount = args.Length;

            if (argCount > 0)
            {
                // some arguments were passed so lets loop through them
                for (argIndex = 0; argIndex < argCount; argIndex++)
                {
                    if (IsValidOption(args[argIndex]))
                    {
                        if (ValueIsRequired(args[argIndex]))
                        {
                            // a value is required for this option
                            if ((argIndex + 1) >= argCount)
                            {
                                // this option requires a value but none was supplied
                                WriteConsoleMessage(args[argIndex] + " requires a value.");
                            }
                            else
                            {
                                if (IsValidOption(args[argIndex + 1]))
                                {
                                    // this option requires a value but the next argument is another option
                                    WriteConsoleMessage(args[argIndex] + " requires a value.");
                                }
                                else
                                {
                                    // this option requires a value so lets get it
                                    if (args[argIndex].Equals(Constants.INITIAL_DIR))
                                    {
                                        _initialDir = args[argIndex + 1];
                                    }
                                }
                            }
                        }
                        else
                        {
                            // no value is required for this option
                            if (args[argIndex].Equals(Constants.HELP))
                            {
                                WriteConsoleMessage(Constants.HELP_TEXT);
                            }

                            if (args[argIndex].Equals(Constants.RECURSIVE))
                            {
                                _recursive = true;
                            }

                            if (args[argIndex].Equals(Constants.LOG_TO_CONSOLE))
                            {
                                _logToConsole = true;
                            }

                            if (args[argIndex].Equals(Constants.DELETE))
                            {
                                _delete = true;
                            }

                            if (args[argIndex].Equals(Constants.HIDDEN))
                            {
                                _hidden = true;
                            }

                            if (args[argIndex].Equals(Constants.CONFIRM_EACH_DELETE))
                            {
                                _confirmEachDelete = true;
                            }
                        }
                    }
                    else
                    {
                        if ((argIndex - 1) >= 0)
                        {
                            if (ValueIsRequired(args[argIndex - 1]) == false)
                            {
                                // a value was supplied for this option when none was required
                                WriteConsoleMessage("Invalid option: " + args[argIndex] + " not recognised.\n");
                            }
                        }
                        else
                        {
                            // this is an unrecognised option
                            WriteConsoleMessage("Invalid option: " + args[argIndex] + " not recognised.\n");
                        }
                    }
                }
            }
            else
            {
                // there were no arguments supplied so lets just show the help text
                WriteConsoleMessage(Constants.HELP_TEXT);
            }
        }


        /// <summary>
        /// Determines whether an optionn is valid.
        /// </summary>
        /// <param name="arg">The option.</param>
        /// <returns>boolean</returns>
        private static bool IsValidOption(string arg)
        {
            bool isValid = false;

            foreach (Option opt in Constants.options)
            {
                if (opt.GetName().Equals(arg))
                {
                    isValid = true;
                    break;
                }
            }

            return isValid;
        }


        /// <summary>
        /// Determines whether the argument requires a value.
        /// </summary>
        /// <param name="arg">The option.</param>
        /// <returns>boolean</returns>
        private static bool ValueIsRequired(string arg)
        {
            bool isRequired = false;

            foreach (Option opt in Constants.options)
            {
                if (opt.GetName().Equals(arg))
                {
                    if (opt.ValueRequired())
                    {
                        isRequired = true;
                        break;
                    }
                }
            }

            return isRequired;
        }


        /// <summary>
        /// Writes a message to the console 
        /// if "silent" option is not set.
        /// </summary>
        /// <param name="message">The string to write to the console.</param></param>
        private static void WriteConsoleMessage(string message)
        {
            if (_silent == false)
            {
                Console.Out.Write(message);
            }
        }
    }
}